@@ -16,27 +16,93 @@
 #
 """Defines the abstract generator."""
 import datetime
+import os
 from abc import ABC
 from abc import abstractmethod
+from pprint import pformat
 
 from faker import Faker
+from jinja2 import ChoiceLoader
+from jinja2 import Environment
+from jinja2 import FileSystemLoader
+from jinja2 import PackageLoader
+from nise.jinja_ext import faker_passthrough
+from nise.util import deepupdate
+from nise.util import load_yaml
+from nise.util import LOG
 
 REPORT_TYPE = "report_type"
+LOADER_LIST = []
 
 
-# pylint: disable=too-few-public-methods
 class AbstractGenerator(ABC):
     """Defines a abstract class for generators."""
 
-    def __init__(self, start_date, end_date):
+    fake = Faker()
+
+    # Jinja template filename defined by each generator
+    TEMPLATE = None
+
+    # keyword args passed to TEMPLATE
+    TEMPLATE_KWARGS = None
+
+    def __init__(self, start_date, end_date, user_config=None):
         """Initialize the generator."""
+        if not self.TEMPLATE:
+            raise AttributeError("Class attribute 'TEMPLATE' must be defined.")
+
+        if not self.TEMPLATE_KWARGS:
+            raise AttributeError("Class attribute 'TEMPLATE_KWARGS' must be defined.")
+
+        LOADER_LIST.append(PackageLoader("nise"))
+        if user_config:
+            LOADER_LIST.append(FileSystemLoader(os.path.abspath(os.path.dirname(user_config))))
+
+        env = Environment(loader=ChoiceLoader(LOADER_LIST))
+        env.globals["faker"] = faker_passthrough
+
+        default_template = env.get_template(self.TEMPLATE)
+        if user_config:
+            user_template = env.get_template(os.path.basename(user_config))
+            user_yaml = load_yaml(user_template.render(**self.TEMPLATE_KWARGS))
+
+            # sort lists of dicts so that generator class names align.
+            generators = user_yaml.get("generators")
+            user_yaml["generators"] = sorted(generators, key=lambda d: list(d.keys()))
+
+            default_yaml = load_yaml(default_template.render(**self.TEMPLATE_KWARGS))
+            config = deepupdate(default_yaml, user_yaml)  # merge user-supplied static file with base template
+        else:
+            config = load_yaml(default_template.render(**self.TEMPLATE_KWARGS))
+
+        # handle special-cases in YAML config syntax
+        config = self._format_config(config)
+
+        # remove top-level class name
+        self.config = []
+        for generators in config.get("generators"):
+            for key, val in generators.items():
+                if key == type(self).__name__:
+                    self.config.append(val)
+
+        LOG.debug("Current config: %s", pformat(self.config))
+
         self.start_date = start_date
         self.end_date = end_date
         self.hours = self._set_hours()
         self.days = self._set_days()
-        self.fake = Faker()
+
         super().__init__()
 
+    @abstractmethod
+    def _format_config(self, config):
+        """Abstract method for sub-classes to handle special cases in the config layout.
+
+        Note for implementors: this method should be used sparingly.
+        When possible, the config file should be designed to minimize special cases that require additional processing.
+        """
+        return config
+
     def _set_hours(self):
         """Create a list of hours between the start and end dates for hourly aws data."""
         hours = []
